/**********************************************************************
* 
* report.h
*
* This file is part of MitoGen
* 
* Copyright (C) 2013-2017 -- Centre for Biomedical Image Analysis (CBIA)
* http://cbia.fi.muni.cz/
* 
* MitoGen is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* MitoGen is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with MitoGen. If not, see <http://www.gnu.org/licenses/>.
* 
* Author: David Wiesner
* 
* Description: Definition of class 'ReportHead' that generates headers
* for REPORTs in multi-threaded implementation of MitoGen.
*
***********************************************************************/


#ifndef MITOGEN_REPORT_H
#define MITOGEN_REPORT_H


#include <iostream>
#include <chrono> // std::chrono::steady_clock
#include <thread>
#include <iomanip> // std::setfill, std::setw


extern thread_local size_t thrID;

// just an alias to make code more readable
typedef std::chrono::steady_clock report_clock;


/**
 * \ingroup toolbox
 *
 * Class handling the REPORT headers in multi-threaded implementation.
 *
 */
class ReportHead
{

public:

	ReportHead() :
		startTime_st(report_clock::now()),
		numOfThreads(0),
		threadCellInfo(NULL) {}
	
	~ReportHead()
	{
		if(numOfThreads != 0)
			delete [] threadCellInfo;
	}
	
	/// set the current number of worker threads
	// NOTE: numOfThreads must be set before worker threads are started
	void SetNumOfThreads(const size_t hwThreads)
	{
		// delete old array
		if ((numOfThreads > 0))
			delete [] threadCellInfo;
		
		// set new size
		numOfThreads = hwThreads;
		
		// if new size is greater than 0, allocate the array
		if (numOfThreads > 0)
		{
			threadCellInfo = new size_t[numOfThreads];
	
			// reset cell IDs
			for(size_t i = 0; i < numOfThreads; ++i)
	   	 		threadCellInfo[i] = SIZE_MAX;
		}
	}
	
	/// set the cell ID processesed by the thread
	// NOTE: SetThreadCellID must be called explicitly in RunFeeder_st
	void SetThreadCellID(const size_t cellID)
	{ threadCellInfo[thrID] = cellID; }
	
	/// print REPORT header	
	void MakeReport() const;
	
	/// methods used by visualization //////////////////////////////////////////
	report_clock::time_point GetStartTime() const
	{ return startTime_st; }

private:

	const report_clock::time_point startTime_st;
	size_t numOfThreads;
	size_t * threadCellInfo;
	
};


#endif
